package info.bukova.isspst.security;

import info.bukova.isspst.Constants;
import info.bukova.isspst.data.Permission;
import info.bukova.isspst.data.PermissionType;
import info.bukova.isspst.data.RequirementBase;
import info.bukova.isspst.data.Role;
import info.bukova.isspst.data.User;
import info.bukova.isspst.data.Workgroup;
import info.bukova.isspst.services.workgroups.WorkgroupService;

import java.util.List;

import org.springframework.security.core.Authentication;

public class RequirementFilterEvaluator implements Evaluator {
	
	private WorkgroupService wgService;
	
	public RequirementFilterEvaluator(WorkgroupService wgService) {
		this.wgService = wgService;
	}

	@Override
	public boolean evaluate(Authentication authentication,
			Object targetDomainObject, String permission) {
		
		RequirementBase req = (RequirementBase) targetDomainObject;
		Workgroup reqWg;
		
		if (!(authentication.getPrincipal() instanceof User)) {
			return false;
		}
		
		User user = (User)authentication.getPrincipal();
		
		Permission appPermission = null;
		for (Permission p : Constants.SPECIAL_PERMISSIONS) {
			if (p.getAuthority().equals(permission)) {
				appPermission = p;
			}
		}
		
		if (appPermission == null) {
			return false;
		}
		
		if (appPermission.getType() == PermissionType.CENTRE) {
			reqWg = req.getCentre();
		} else {
			reqWg = req.getWorkgroup();
		}
		
		if (wgService.isMember(reqWg, user)) {
			List<Role> roles = wgService.getUserWorkgroupRoles(reqWg, user);
			for (Role r : roles) {
				for (Permission p : r.getPermissions()) {
					if (p.getAuthority().equals(appPermission.getAuthority())) {
						return true;
					}
				}
			}
		}
		
		return false;
	}

}
